<?php
class PullModeConfig extends CFormModel {
	
	const PULL_DISABLED  	= "disabled";
	const PULL_STATIC     	= "static";
	const PULL_FILE     	= "file";
	const PULL_REMOTE     	= "remote";
	
    protected $config_name='/etc/raperca/uploaderconf.xml';
    protected $uploadercal="/etc/raperca/uploader.ics";
    
    protected $logconf="/etc/raperca/uploaderlog.xml";
    
    var $selectMode=null;
    
    var $enable_project=0;
    var $project_source="";
    var $hourProj=0;
    var $minProj=0;
    var $enable_log=0;
    var $log_source="";
    var $hourLog=0;
    var $minLog=0;
    var $logs='accounting';
    
    var $icalfile="";
    var $icsfile;
    
    var $backup = false;
    
    var $uri="";
    
    var $checktime = "24h";
    
    var $priority=null;
    
    var $https_useDefaultCA=null;
    
    var $priorityList = array(  'trace'=>"Trace",
                                'debug'=>"Debug",
                                'info'=>"Info",
                                'warn'=>"Warning",
                                'error'=>"Error",
                                'fatal'=>"Fatal",);
    
    public function rules() {
		$res = array(
            array( "enable_project, hourProj, minProj, timeProj", 'safe', 'on'=>'manual' ),
            array( "enable_log, hourLog, minLog, timeLog, logs", 'safe', 'on'=>'manual' ),
            array( "project_source, log_source", 'url', 'allowEmpty'=>true, 'pattern'=>'/^(http|https):\/\/(([A-Z0-9][A-Z0-9_-]*)(\.[A-Z0-9][A-Z0-9_-]*)*)/i', 'on'=>'manual' ),
			array( "project_source, log_source", 'emptyselect', 'on'=>'manual' ),
            
            array( "icalfile", 'file', 'types'=>'ics', 'on'=>'file' ),
            
            array( "icsfile", 'safe', 'on'=>'file_inline' ),
            
            array( "icalSourceHref, checktime", 'safe', 'on'=>'remote' ),
            array( "uri", 'url', 'pattern'=>'/^(http|https):\/\/(([A-Z0-9][A-Z0-9_-]*)(\.[A-Z0-9][A-Z0-9_-]*)*)/i', 'on'=>'remote' ),
            array( "priority", 'safe' ),
    	);
        return $res;
	}
    public function attributeLabels()
	{
        $labels = array(
            'mode_dis'=>'Disabled',
            'mode_manual'=>'Manual settings',
            'mode_file'=>'From uploaded iCalendar file (ics)',
            'mode_remote'=>'From remote iCalendar file (ics)',
            'enable_project'=>'Automatically upload project to the '.Yii::app()->user->branding->product,
            'enable_log'=>'Automatically upload logs from the '.Yii::app()->user->branding->product,
            'project_source'=>'Project source',
            'hourProj'=>'Time',
            'log_source'=>'Log destination',
            'hourLog'=>'Time',
            'logs_accounting'=>'Accounting logs only',
            'logs_all'=>'All logs',
            'icalfile'=>'Schedule file',
            'uri'=>'Schedule uri',
            'checktime'=>'Check calendar every',
            'icalSourceHref'=>"Use '[serial]' in the URI to use the serial number of the ".Yii::app()->user->branding->product,
            'priority'=>'Log level',
		);
        if ( $this->hasFusionRestrictions() ) {
            $labels['enable_project'] = 'Automatically upload project to the '.Yii::app()->user->branding->product.'\'s /publish folder';
        }
        return $labels;
	}
    
    function getTimeProj (){
        return $this->hourProj.":".$this->minProj;
    }
    function setTimeProj ( $time ){
        $times = explode(":", $time);
        if ( count($times==2) ){
            $this->hourProj = $times[0];
            $this->minProj = $times[1];
        }
    }
    function getTimeLog (){
        return $this->hourLog.":".$this->minLog;
    }
    function hasFusionRestrictions () {
        $fusion = new FusionSetup;
        return $fusion->active;
    }
    
    function setTimeLog( $time ){
        $times = explode(":", $time);
        if ( count($times==2) ){
            $this->hourLog = $times[0];
            $this->minLog = $times[1];
        }
    }
    function setSelectMode( $mode ) {
        $this->selectMode = $mode;
        switch ($mode) {
            case PullModeConfig::PULL_DISABLED: $this->setScenario('disabled'); break;
            case PullModeConfig::PULL_STATIC: $this->setScenario('manual'); break;
            case PullModeConfig::PULL_FILE: if ($this->backup)
                                                $this->setScenario('file_inline'); 
                                            else
                                                $this->setScenario('file'); 
                                            break;
            case PullModeConfig::PULL_REMOTE: $this->setScenario('remote'); break;
            default: $this->setError('selectMode', 'Mode not supported');
        }
    }
    function setIcalSourceHref( $href ) {
        $this->uri=preg_replace("@\[serial\]@",Yii::app()->user->info->serial, $href);
    }
    function getIcalSourceHref() {
        return preg_replace("@".Yii::app()->user->info->serial."@","[serial]", $this->uri);
    }
    
    function load()
    {
        $spxNS = 'http://www.spinetix.com/namespace/1.0/spxconf';
        $xlinkNS = 'http://www.w3.org/1999/xlink';


        $config = new DOMDocument;
        $config->loadXML(file_get_contents($this->config_name));
        $this->selectMode = PullModeConfig::PULL_DISABLED;
        
        
        $schedule=$config->getElementsByTagName("schedule");
        if ($schedule->length>0) {
            $link=$schedule->item(0)->getAttributeNS($xlinkNS,"href");
            if ( $link=="uploader.ics" || $link==$this->uploadercal ) {
                $this->selectMode = PullModeConfig::PULL_FILE;
            } else {
                $this->selectMode = PullModeConfig::PULL_REMOTE;
                $this->uri=$link;
                $checktime = $schedule->item(0)->getAttribute("check");
                if ($checktime=="") 
                    $this->checktime = "24h";
                else {
                    $this->checktime = Yii::app()->user->tools->secs_to_timestr($checktime);
                }
            }
        } else {
            // manual
            $proj=$config->getElementsByTagName("action");
            foreach ($proj as $action) {
                $name=$action->getAttribute("summary");
                if ($name=='publish') {
                    $this->enable_project = 1;
                    $this->project_source=$action->getAttributeNS($xlinkNS,"href");
                    $time = $action->getAttribute("time");
                    $this->hourProj=floor($time/3600);
                    $this->minProj=floor(($time-$this->hourProj*3600)/60);
                    $this->selectMode = PullModeConfig::PULL_STATIC;
                }
                if ($name=='upload') {
                    $this->enable_log = 1;
                    $this->log_source=$action->getAttributeNS($xlinkNS,"href");
                    $time = $action->getAttribute("time");
                    $this->hourLog=floor($time/3600);
                    $this->minLog=floor(($time-$this->hourLog*3600)/60);
                    $this->logs=$action->getAttribute("type");
                    $this->selectMode = PullModeConfig::PULL_STATIC;
                }
            }
        }
        $https_configs=$config->getElementsByTagName("https");
        foreach ($https_configs as $https) {
            if ( $https->getAttribute("useDefaultCA")=='true' )
                $this->https_useDefaultCA = true;
            else
                $this->https_useDefaultCA = false;
        }
        
        $logconfig = new DOMDocument;
        $logconfig->loadXML(file_get_contents($this->logconf));
        $priority=$logconfig->getElementsByTagName("priority");
        if ( $priority->length>0 )
            $this->priority = $priority->item(0)->getAttribute("value");
    }

    function save()
    {
        $spxNS = 'http://www.spinetix.com/namespace/1.0/spxconf';
        $xlinkNS = 'http://www.w3.org/1999/xlink';

        if ( $this->selectMode!=null ) {
            $config = new DOMDocument;
            $main=$config->appendChild($config->createElementNS($spxNS,'uploader'));
            
            if ( $this->https_useDefaultCA !== null ) {
                $https = $main->appendChild( $config->createElementNS($spxNS,'https') );
                if ( $this->https_useDefaultCA ) {
                    $https->setAttribute( "useDefaultCA", 'true' );
                } else {
                    $https->setAttribute( "useDefaultCA", 'false' );
                }
            }
            switch ( $this->selectMode ) {
                case PullModeConfig::PULL_DISABLED:// disabled
                    $proj=$main->appendChild($config->createElementNS($spxNS,'disabled'));
                break;
                case PullModeConfig::PULL_STATIC:// manual setting
                    $saveTime=-1;
                    if ( $this->enable_project ){
                        $proj=$main->appendChild($config->createElementNS($spxNS,'action'));
                        $proj->setAttribute("summary", 'publish');
                        $proj->setAttribute("guid", uniqid() );
                        $proj->setAttributeNS($xlinkNS,"xlink:href", $this->project_source);
                        $saveTime = $this->hourProj*3600+$this->minProj*60;
                        $proj->setAttribute("time", $saveTime);
                        if ( $this->hasFusionRestrictions() ) {
                            $desc=$proj->appendChild($config->createElementNS($spxNS,'desc'));
                            $desc->appendChild($config->createTextNode("-dest /publish/"));    
                        }
                    }
                    if ( $this->enable_log ){
                        $proj=$main->appendChild($config->createElementNS($spxNS,'action'));
                        $proj->setAttribute("summary", 'upload');
                        $proj->setAttribute("guid", uniqid() );
                        $proj->setAttributeNS($xlinkNS,"xlink:href", $this->log_source);
                        $saveTime2 = $this->hourLog*3600+$this->minLog*60;
                        $proj->setAttribute("time", $saveTime2);
                        
                        $proj->setAttribute("type", $this->logs);
                        if ( $this->logs=='accounting' ) {
                            $desc=$proj->appendChild($config->createElementNS($spxNS,'desc'));
                            $desc->appendChild($config->createTextNode("-accounting"));
                        } else {
                            $desc=$proj->appendChild($config->createElementNS($spxNS,'desc'));
                            $desc->appendChild($config->createTextNode("-logs"));
                        }
                    }
                break;
                case PullModeConfig::PULL_FILE:// file
                    if ( $this->scenario == 'file_inline' ){
                        $this->setSchedule( $this->icsfile );
                    } else if ( $this->scenario != 'fusion' ){
                        $obj = CUploadedFile::getInstance($this,'icalfile');
                        if( $obj===null || !$obj->saveAs( $this->uploadercal ) ) {
                            $this->addError('uploadercal', 'Saving the ics file failed');
                        }
                    }
                    $proj=$main->appendChild($config->createElementNS($spxNS,'schedule'));
                    $proj->setAttributeNS($xlinkNS,"xlink:href", "uploader.ics");
                break;
                case PullModeConfig::PULL_REMOTE:// remote
                    $proj=$main->appendChild($config->createElementNS($spxNS,'schedule'));
                    
                    $proj->setAttributeNS($xlinkNS,"xlink:href", $this->uri);
                    $time = Yii::app()->user->tools->timestr_to_secs($this->checktime);
                    if ( $time===false )
                        $this->addError("checktime", "Invalid time format");
                    if ( $time<10 ) {
                        $time = 10;
                        $this->checktime = "10s";
                    }
                    $proj->setAttribute("check", $time);
                break;
            }    
            
            if ( !$this->hasErrors() ){
                Yii::app()->user->tools->save_file($this->config_name,$config->saveXML());
                exec('/etc/init.d/uploader reload');
		    }
        }
        
            
        if ( $this->priority!=null ) {
            $logconfig = new DOMDocument;
            $logconfig->loadXML(file_get_contents($this->logconf));
            $priority=$logconfig->getElementsByTagName("priority");
            if ( $priority->length>0) {
                $priority->item(0)->setAttribute("value",$this->priority);
                Yii::app()->user->tools->save_file($this->logconf, $logconfig->saveXML());
                exec('/etc/init.d/uploader reload');
            }else 
                $this->setError('priority', 'Cannot save priority');
            
        }        
    }
    function setSchedule( $cnt ) {
        file_put_contents( $this->uploadercal, $cnt );
    }
    function getSchedule() {
        return file_get_contents($this->uploadercal);
    }
    function download() {
        if (file_exists($this->uploadercal) ){
            header("Content-type: text/calendar");
            header('Content-Disposition: attachment; filename="pullmode.ics"');
	        echo file_get_contents($this->uploadercal);
            return true;            
        } else {
            return false;
        }
    }
	function emptyselect( $attribute,$params ) {
		if ( ( $attribute=='project_source' && $this->project_source=="" && $this->enable_project) ||
			 ( $attribute=='log_source' && $this->log_source=="" && $this->enable_log) ){
			// error
			$this->addError($attribute, Yii::t('yii','{attribute} is not a valid URL.', array("{attribute}"=>$this->getAttributeLabel($attribute))) );
		}		
	}
}